<?php

/**
 * JHijri class file.
 *
 * @author jerry2801 <jerry2801@gmail.com>
 * @version alpha1
 *
 * A typical usage of JHijri is as follows:
 * <pre>
 * $this->widget('application.extensions.hijri.JHijri', array(
 *     'id'=>Html::activeId($model,'post_date'), // Gregorian Date Field Id
 *     'value'=>$model->postDateFormatByHijri,
 * ));
 * </pre>
 */

class JHijri extends CWidget
{
    public $id;
    public $value;
    public $htmlOptions=array();

    public function init()
    {
        if(!isset($this->htmlOptions['id']))
            $this->htmlOptions['id']=$this->id.'_hijri';
    }

    public function run()
    {
        $id=$this->id;
        echo CHtml::button('< To Gregorian',array('id'=>'toGregorianBtn_'.$id));
        echo CHtml::button('To Hijri >',array('id'=>'toHijriBtn_'.$id));
        echo CHtml::textField(null,$this->value,$this->htmlOptions);

		$cs = Yii::app()->getClientScript();
        $cs->registerScript(__CLASS__.'_'.$id,$this->getScript());
    }

    public function getScript()
    {
        $id=$this->id;
        $hijriId=$this->htmlOptions['id'];
        return <<< EOF

function validate(dataStr)
{
	var dArry=dataStr.split('/');
	var d=parseNumber(dArry[0]);
	var m=parseNumber(dArry[1]);
	var y=parseNumber(dArry[2]);

	if(d>0&&d<=30&&m>0&&m<=12&&y>0)
		return true;
    else
		return false;
}

function intPart(floatNum)
{
    if (floatNum< -0.0000001)
        return Math.ceil(floatNum-0.0000001)
    return Math.floor(floatNum+0.0000001)
}

function parseNumber(numStr)
{
	var temp = numStr.substring(0,1); 
	if(temp=='0')
		return parseInt(numStr.substring(1));
	else
		return parseInt(numStr);
}

function toHijri(dataStr)
{
    if(validate(dataStr))
    {
        var dArry=dataStr.split('/');
        var d=parseNumber(dArry[0]);
        var m=parseNumber(dArry[1]);
        var y=parseNumber(dArry[2]);
        var delta=0;

        if ((y>1582)||((y==1582)&&(m>10))||((y==1582)&&(m==10)&&(d>14)))
        {
            //added delta=0 on jd to comply isna rulling 2007
            jd=intPart((1461*(y+4800+intPart((m-14)/12)))/4)+intPart((367*(m-2-12*(intPart((m-14)/12))))/12)-
            intPart( (3* (intPart(  (y+4900+    intPart( (m-14)/12)     )/100)    )   ) /4)+d-32075+delta
        }
        else
        {
            //added +1 on jd to comply isna rulling
            jd = 367*y-intPart((7*(y+5001+intPart((m-9)/7)))/4)+intPart((275*m)/9)+d+1729777+delta
        }
        //arg.JD.value=jd
        //added -1 on jd1 to comply isna rulling
        jd1=jd-delta
        //arg.wd.value=weekDay(jd1%7)
        l=jd-1948440+10632
        n=intPart((l-1)/10631)
        l=l-10631*n+354
        j=(intPart((10985-l)/5316))*(intPart((50*l)/17719))+(intPart(l/5670))*(intPart((43*l)/15238))
        l=l-(intPart((30-j)/15))*(intPart((17719*j)/50))-(intPart(j/16))*(intPart((15238*j)/43))+29
        m=intPart((24*l)/709)
        d=l-intPart((709*m)/24)
        y=30*n+j-30

        return d+'/'+m+'/'+y;
    }
    else
    {
        alert('Please input correct date!');
        return '';
    }
}

function toGregorian(dataStr)
{
    if(validate(dataStr))
    {
        var dArry=dataStr.split('/');
        var d=parseNumber(dArry[0]);
        var m=parseNumber(dArry[1]);
        var y=parseNumber(dArry[2]);
        var delta=0
        //added - delta=0 on jd to comply isna rulling
        jd=intPart((11*y+3)/30)+354*y+30*m-intPart((m-1)/2)+d+1948440-385-delta
        //arg.JD.value=jd
        //arg.wd.value=weekDay(jd%7)
        if (jd> 2299160 )
        {
            l=jd+68569
            n=intPart((4*l)/146097)
            l=l-intPart((146097*n+3)/4)
            i=intPart((4000*(l+1))/1461001)
            l=l-intPart((1461*i)/4)+31
            j=intPart((80*l)/2447)
            d=l-intPart((2447*j)/80)
            l=intPart(j/11)
            m=j+2-12*l
            y=100*(n-49)+i+l
        }
        else
        {
            j=jd+1402
            k=intPart((j-1)/1461)
            l=j-1461*k
            n=intPart((l-1)/365)-intPart(l/1461)
            i=l-365*n+30
            j=intPart((80*i)/2447)
            d=i-intPart((2447*j)/80)
            i=intPart(j/11)
            m=j+2-12*i
            y=4*k+n+i-4716
        }

        if(d<10)
            d='0'+d;

        if(m<10)
            m='0'+m;

        return d+'/'+m+'/'+y;
    }
    else
    {
        alert('Please input correct date!');
        return '';
    }
}


$('#toGregorianBtn_{$id}').bind('click',function() {
    $('#{$id}').val( toGregorian($('#{$hijriId}').val()) );
});

$('#toHijriBtn_{$id}').bind('click',function() {
    $('#{$hijriId}').val( toHijri($('#{$id}').val()) );
});
EOF;
    }
}